/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "newprofiledialog.h"
#include <kcombobox.h>
#include <klineedit.h>
#include <qcheckbox.h>
#include <kurlrequester.h>
#include <kmessagebox.h>
#include <qlistbox.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <knuminput.h>
#include <qobject.h>
#include <qtextedit.h>


#include "networkinterface.h"
#include "importp12certificatedialog.h"
#include "advancedsettingsdialog.h"
#include "vpnaccountdata.h"

NewProfileDialog::NewProfileDialog( QPtrList<VpnAccountData> *AccountList, QTextEdit *LogOutput )
		: NewProfileDialogbase() {

	this->AccountList = AccountList;
	this->LogOutput = LogOutput;


	connect( ConnectionTypeComboBox, SIGNAL( activated( const QString& ) ), this, SLOT( connectiontypeChanged( const QString& ) ) );
	connect( AuthTypeComboBox , SIGNAL( activated( const QString& ) ), this, SLOT( authTypeChanged( const QString& ) ) );
	connect( buttonCancel , SIGNAL( clicked () ), this, SLOT( reject() ) );
	connect( ImportCertificatePushButton, SIGNAL( clicked () ), this, SLOT( importCertificate() ) );
	connect( AdvancedSettingsPushButton, SIGNAL( clicked () ), this, SLOT( advancedSettingsClicked() ) );
	newProfileCreated = false;
	QString Name = "";

	// currently static for new profile
	VpnAccountData::ConnectionType ConnType = VpnAccountData::ConnectionType(VpnAccountData::other);
	Gateway = "";
	ID = "";
	Username = "";
	UserPassword = "";
	GroupPassword = "";
	saveUserPassword = false;
	saveGroupPassword = false;

	ApplicationVersion = "";
	Port = 500;
	NtDomainName = "";
	PerfectForwardSecrecy = "";
	IkeGroup = "";
	useApplicationVersion = false;
	useGlobalIpsecSecret = false;
	useIkeGroup = false;
	useLocalPort = false;
	useNtDomainName = false;
	useSingleDes = false;
	useAdvancedSettings = false;
	usePerfectSecrecy = false;
	


	profileData = new VpnAccountData( ConnType, Name, Gateway, ID, GroupPassword, Username,
	                                  UserPassword, saveUserPassword, saveGroupPassword, ApplicationVersion, Port, NtDomainName, PerfectForwardSecrecy, IkeGroup, useApplicationVersion, useGlobalIpsecSecret, useIkeGroup, useLocalPort, useNtDomainName, useSingleDes, useAdvancedSettings, usePerfectSecrecy );


	NetworkInterface iface;
	QStringList list = iface.getAllNetworkInterfaces();
	NetworkDeviceComboBox->insertItem( "default" );
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
		NetworkDeviceComboBox->insertItem( QString( *it ) );
	}
	NetworkDeviceComboBox->setEnabled( true );
	NetworkDeviceComboBox->setCurrentText( "default" );

	certpathURLRequester->setURL( "/etc/racoon/certs" );

	ConnectionTypeComboBox->setCurrentItem( 2 ); // other
	buttonOk->setEnabled( false );
	
	AuthTypeComboBox->setCurrentItem(0); // x509 cert
	
	authTypeChanged( "" );
	connectiontypeChanged( "" );
}

NewProfileDialog::~NewProfileDialog() {}

void NewProfileDialog::accept() {
	bool canSave = true;
	if ( ConnectionTypeComboBox->currentItem() == 0 )
		profileData->setConnectionType( VpnAccountData::cisco );
	else if ( ConnectionTypeComboBox->currentItem() == 1 ) {
		profileData->setConnectionType( VpnAccountData::racoon );

		bool validAddr = true;
		if ( RemoteNetAddrLineEdit->text().contains( '.' ) != 3 ) {
			KMessageBox::error ( this, i18n( "No IP address entered!" ), i18n( "No IP address" ) );
			validAddr = false;
		} else {
			QString addr = RemoteNetAddrLineEdit->text();
			int part0 = addr.section( '.', 0, 0 ).toInt();
			int part1 = addr.section( '.', 1, 1 ).toInt();
			int part2 = addr.section( '.', 2, 2 ).toInt();
			int part3 = addr.section( '.', 3, 3 ).toInt();

			if ( ( part0 < 1 || part0 > 254 ) || ( part1 < 1 || part1 > 254 ) || ( part2 < 1 || part2 > 254 ) || ( part3 < 0 || part3 > 254 ) ) {
				KMessageBox::error ( this, i18n( "IP address is invalid!" ), i18n( "Invalid IP address" ) );
				validAddr = false;
				canSave = false;
			}
			if ( validAddr == true ) {
				profileData->setRemoteNet( addr + "/" + QString().setNum( RemoteNetMaskSpinBox->value() ) );
			}

		}

		if (AuthTypeComboBox->currentItem() == 0)
		    profileData->setAuthType( VpnAccountData::cert );
		else
		    profileData->setAuthType( VpnAccountData::psk );
		
		profileData->setX509Certificate( x509certURLRequester->url() );
		profileData->setCertPath( certpathURLRequester->url() );

	} else
		profileData->setConnectionType( VpnAccountData::other );

	profileData->setSaveGroupPassword( CheckGroupPass->isChecked() );
	profileData->setSaveUserPassword( CheckUserPass->isChecked () );
	profileData->setID( IDLineEdit->text() );
	profileData->setUserName( UsernameLineEdit->text () );
	profileData->setGroupPassword( GroupPasswordEdit->text( ) );
	profileData->setUserPassword( PasswordEdit->text ( ) );
	profileData->setGateway( gatewayLineEdit->text() );
	profileData->setPreSharedKey( PSKLineEdit->text() );
	profileData->setX509Certificate( x509certURLRequester->url() );
	profileData->setCertPath( certpathURLRequester->url() );
	profileData->setNetworkDevice( NetworkDeviceComboBox->currentText() );
	if (AuthTypeComboBox->currentItem() == 0)
	    profileData->setAuthType( VpnAccountData::cert );
	else
	    profileData->setAuthType( VpnAccountData::psk );
	if ( canSave ) {

		bool nameOk = true;
		QString Name = ProfileLineEdit->text();

		if (Name.contains(' ')){
			KMessageBox::error ( this, i18n( "Profile name cant contain spaces!" ), i18n( "Spaces not allowed" ) );
			nameOk = false;
		}
		
		VpnAccountData * it;
		for ( it = AccountList->first(); it; it = AccountList->next() ) {
			if ( it->getName() == Name ) {
				KMessageBox::error ( this, i18n( "Profile name exists!" ), i18n( "Name exist" ) );
				nameOk = false;
				break;
			}
		}

		if ( nameOk ) {
			profileData->setName(Name);
			AccountList->append( profileData );
			newProfileCreated = true;
		QDialog::accept();
		}
		
	}
}

void NewProfileDialog::importCertificate() {
	ImportP12CertificateDialog importdlg( LogOutput );
	//int result =
	importdlg.exec();
}


void NewProfileDialog::connectiontypeChanged( const QString& ) {

	// First we should set a type
	buttonOk->setEnabled( FALSE );
	AdvancedSettingsPushButton->setEnabled( false );

	int item = ConnectionTypeComboBox->currentItem();

	if ( item == 0 ) {
		LogOutput->append( "new type: cisco" );
		CheckGroupPass->setEnabled( true );
		CheckUserPass->setEnabled( true );
		IDLineEdit->setEnabled( true );
		UsernameLineEdit->setEnabled( true );
		GroupPasswordEdit->setEnabled( true );
		PasswordEdit->setEnabled( true );
		AdvancedSettingsPushButton->setEnabled( true );

		x509certURLRequester->setEnabled( false );
		certpathURLRequester->setEnabled( false );
		NetworkDeviceComboBox->setEnabled( false );
		AuthTypeComboBox->setEnabled( false );
		PSKLineEdit->setEnabled( false );
		buttonCancel->setEnabled( TRUE );
		buttonOk->setEnabled( TRUE );
		AdvancedSettingsPushButton->setEnabled( true );
		RemoteNetAddrLineEdit->setEnabled( false );
		RemoteNetMaskSpinBox->setEnabled( false );
	} // vpnc

	else if ( item == 1 ) {
		LogOutput->append( "new type: racoon" );
		//     KURLRequester* x509certURLRequester;
		//     KComboBox* AuthTypeComboBox;
		//     KPushButton* ImportCertificatePushButton;
		//     KLineEdit* PSKLineEdit;

		x509certURLRequester->setEnabled( true );
		certpathURLRequester->setEnabled( true );
		NetworkDeviceComboBox->setEnabled ( true );
		AuthTypeComboBox->setEnabled( true );
		PSKLineEdit->setEnabled( true );
		RemoteNetAddrLineEdit->setEnabled( true );
		RemoteNetMaskSpinBox->setEnabled( true );
		PasswordEdit->setEnabled( false );
		CheckGroupPass->setEnabled( false );
		CheckUserPass->setEnabled( false );
		IDLineEdit->setEnabled( false );
		GroupPasswordEdit->setEnabled( false );
		PasswordEdit->setEnabled( false );
		UsernameLineEdit->setEnabled( false );
		AdvancedSettingsPushButton->setEnabled( false );

		buttonCancel->setEnabled( TRUE );
		buttonOk->setEnabled( TRUE );
		AdvancedSettingsPushButton->setEnabled( true );
	} // racoon

	// other
	else {
		LogOutput->append( "new type: other" );
		// 		x509certURLRequester->setEnabled( false );
		// 		AuthTypeComboBox->setEnabled( false );
		// 		PSKLineEdit->setEnabled( false );
		//
		// 		CheckGroupPass->setEnabled( false );
		// 		CheckUserPass->setEnabled( false );
		// 		IDLineEdit->setEnabled( false );
		// 		UsernameLineEdit->setEnabled( false );
		// 		GroupPasswordEdit->setEnabled( false );
		// 		AdvancedSettingsPushButton->setEnabled( false );
//		buttonCancel->setEnabled( FALSE );
		buttonOk->setEnabled( FALSE );
	}
}

void NewProfileDialog::authTypeChanged( const QString& ) {
	if ( AuthTypeComboBox->currentItem() == 0 ) {
		// certificate
		PSKLineEdit->setEnabled( false );
		x509certURLRequester->setEnabled( true );
		certpathURLRequester->setEnabled( true );
	}
	if ( AuthTypeComboBox->currentItem() == 1 ) {
		// psk
		PSKLineEdit->setEnabled( true );
		x509certURLRequester->setEnabled( false );
		certpathURLRequester->setEnabled( false );
	}

}

void NewProfileDialog::reject() {
	QDialog::reject();
}

void NewProfileDialog::profileNameChanged( const QString& ) {
	// 	LogOutput->append ( "text changed!" );
	if ( ProfileLineEdit->text().isEmpty() )
		buttonOk->setEnabled( false );
	else
		buttonOk->setEnabled( true );
}

void NewProfileDialog::advancedSettingsClicked() {
	AdvancedSettingsDialog d1;

	d1.AdvancedSettingsCheckbox->setChecked ( false );
	d1.UseGlobalIpsecSecretCheckbox->setChecked ( false );
	d1.SingleDesCheckbox->setChecked( false );
	d1.IkeGroupCheckbox->setChecked ( false );
	d1.LocalPortCheckbox->setEnabled ( false );
	d1.PerfectForwardSecurityCheckbox->setChecked( false );
	d1.NtDomainNameCheckbox->setEnabled ( false );
	d1.ApplicationVersionCheckbox->setEnabled ( false );
	d1.UseGlobalIpsecSecretCheckbox->setEnabled ( false );

	d1.LocalPortSpinbox->setValue( profileData->getLocalPort() );
	d1.IkeGroupCombobox->setCurrentText( profileData->getIkeGroup() );
	d1.PerfectForwardSecrecyCombobox->setCurrentText( profileData->getPerfectForwardSecrety() );
	d1.NtDomainNameLineedit->setText( profileData->getNtDomainName() );
	d1.ApplicationVersionLineedit->setText( profileData->getApplicationVersion() );

	if ( profileData->getUseAdvancedSettings() ) {
		d1.LocalPortCheckbox->setEnabled ( true );
		d1.IkeGroupCheckbox->setEnabled ( true );
		d1.PerfectForwardSecurityCheckbox->setEnabled ( true );
		d1.SingleDesCheckbox->setEnabled ( true );
		d1.NtDomainNameCheckbox->setEnabled ( true );
		d1.ApplicationVersionCheckbox->setEnabled ( true );
		d1.UseGlobalIpsecSecretCheckbox->setEnabled ( true );
		d1.AdvancedSettingsCheckbox->setChecked ( true );

		if ( profileData->getUseSingleDes() ) {
			d1.SingleDesCheckbox->setChecked( true );
		}

		if ( profileData->getUseIkeGroup() ) {
			d1.IkeGroupCheckbox->setChecked ( true );
		}

		if ( profileData->getUseGlobalIpsecSecret() ) {
			d1.UseGlobalIpsecSecretCheckbox->setChecked ( true );
		}

		if ( profileData->getUsePerfectForwardSecrety() ) {
			d1.PerfectForwardSecrecyCombobox->setEnabled ( true );
			d1.PerfectForwardSecurityCheckbox->setChecked( true );
		}

		if ( profileData->getUseNtDomainName() ) {
			d1.NtDomainNameLineedit->setEnabled ( true );
			d1.NtDomainNameCheckbox->setChecked( true );
		}

		if ( profileData->getUseApplicationVersion() ) {
			d1.ApplicationVersionLineedit->setEnabled ( true );
			d1.ApplicationVersionCheckbox->setChecked( true );
		}

		if ( profileData->getUseLocalPort() ) {
			d1.LocalPortSpinbox->setEnabled ( true );
			d1.LocalPortCheckbox->setChecked( true );
		}

		if ( profileData->getUseIkeGroup() ) {
			d1.IkeGroupCombobox->setEnabled ( true );
		}

	}


	d1.exec();

	if ( d1.AdvancedSettingsCheckbox->isChecked() ) {
		profileData->setUseAdvancedSettings( true );
	} else {
		profileData->setUseAdvancedSettings( false );
	}
	if ( d1.NtDomainNameCheckbox->isChecked() ) {
		profileData->setNtDomainName( d1.NtDomainNameLineedit->text() );
		profileData->setUseNtDomainName( true );
	} else {
		profileData->setUseNtDomainName( false );
	}

	if ( d1.ApplicationVersionCheckbox->isChecked() ) {
		profileData->setApplicationVersion( d1.ApplicationVersionLineedit->text() );
		profileData->setUseApplicationVersion( true );
	} else {
		profileData->setUseApplicationVersion( false );
	}

	if ( d1.IkeGroupCheckbox->isChecked() ) {
		profileData->setIkeGroup( d1.IkeGroupCombobox->currentText() );
		profileData->setUseIkeGroup( true );
	} else {
		profileData->setUseIkeGroup( false );
	}

	if ( d1.PerfectForwardSecurityCheckbox->isChecked() ) {
		profileData->setPerfectForwardSecrety( d1.PerfectForwardSecrecyCombobox->currentText() );
		profileData->setUsePerfectForwardSecrety( true );
	} else {
		profileData->setUsePerfectForwardSecrety( false );
	}


	if ( d1.SingleDesCheckbox->isChecked() ) {
		profileData->setUseSingleDes( true );
	} else {
		profileData->setUseSingleDes( false );
	}

	if ( d1.UseGlobalIpsecSecretCheckbox->isChecked() ) {
		profileData->setUseGlobalIpsecSecret( true );
	} else {
		profileData->setUseGlobalIpsecSecret( false );
	}


	if ( d1.LocalPortCheckbox->isChecked() ) {
		profileData->setLocalPort( d1.LocalPortSpinbox->value() );
		profileData->setUseLocalPort( true );
	} else {
		profileData->setUseLocalPort( false );
	}
}


#include "newprofiledialog.moc"
