/***************************************************************************
*   Copyright (C) 2004 by Christoph Thielecke                             *
*   crissi99@gmx.de                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef VPNACCOUNTDATA_H
#define VPNACCOUNTDATA_H

#include <qstring.h>
#include <qobject.h>

//enum connectiontypes{vpnc,racoon,other};



/**
* holds the data for a account
* @author Christoph Thielecke
*/
class VpnAccountData : public QObject {
	public:
		enum ConnectionType {cisco, racoon, freeswan, other};
		enum AuthenticationType {cert, psk};

		/**
		 * Konstruktor
		 * @param ConnType Type (cisco, freeswan,racoon, etc)
		 * @param Name The name of the account
		 * @param Gateway The hostname/IP of the VPN gateway
		 * @param ID The ID of the VPN (cisco)
		 * @param GroupPassword The IPSec group secret (cisco)
		 * @param Username The username (cisco)
		 * @param UserPassword The password of the user (cisco)
		 * @param saveUserPassword True if the password of user should be stored (cisco)
		 * @param saveGroupPassword True if the IPSec group secret should be stored (cisco)
		 * @param ApplicationVersion Userdefinied application version string (cisco)
		 * @param Port Userdefinied port where vpnc should listen (cisco)
		 * @param NtDomainName Domain name of NT server for authention (optional) (cisco)
		 * @param PerfectForwardSecrecy What PFS should be used (cisco)
		 * @param IkeGroup Which IKE group should be used (cisco)
		 * @param useApplicationVersion True if userdefinied application version should be sent (cisco)
		 * @param useGlobalIpsecSecret True if the global IPSec secret from /etc/vpnc/default.conf should be used (cisco)
		 * @param useIkeGroup True if userdefinied IKE should be used (cisco)
		 * @param useLocalPort True if userdefinied port should be used (cisco)
		 * @param useNtDomainName True if NT domain name should be used for authentication (cisco)
		 * @param useSingleDes True if single DES should be used (cisco)
		 * @param useAdvancedSettings True if some advanced settings should be enabled (cisco)
		 * @param usePerfectSecrecy True if userdefinied PFS should be used  (cisco)
		 * @return nothing
		 */
		VpnAccountData( ConnectionType ConnType, QString Name, QString Gateway, QString ID,
		                QString GroupPassword, QString Username, QString UserPassword, bool saveUserPassword,
		                bool saveGroupPassword, QString ApplicationVersion, int Port, QString NtDomainName, QString PerfectForwardSecrecy, QString IkeGroup, bool useApplicationVersion, bool useGlobalIpsecSecret, bool useIkeGroup, bool useLocalPort, bool useNtDomainName, bool useSingleDes, bool useAdvancedSettings , bool usePerfectSecrecy );

		/**
		* Destruktor
		*/
		~VpnAccountData();

		QString getName(); //< returns Name
		ConnectionType getConnectionType(); //< returns Connection type
		QString getGateway(); //< returns Gateway
		QString getID(); //< returns ID
		QString getGroupPassword(); //< returns GroupPassword
		QString getUserName(); //< returns UserName
		QString getUserPassword(); //< returns UserPassword
		QString getIkeGroup(); //< returns IkeGroup
		QString getPerfectForwardSecrety(); //< returns PerfectForwardSecrecy
		QString getNtDomainName(); //< returns NtDomainName
		QString getApplicationVersion(); //< returns ApplicationVersion
		QString getRemoteNet(); //<returns remoteNet
		QString getPreSharedKey(); // returns PreSharedKey
		QString getX509Certificate(); //< returns x509Certificate
		QString getNetworkDevice(); //< returns NetworkDevice
		QString getCertPath(); //<returns CertPath
		QString getSpecialRemoteID(); //< retunr SpecialRemoteID;
		AuthenticationType getAuthType(); //< returns AuthType
		bool getSaveUserPassword(); //< returns saveUserPassword
		bool getSaveGroupPassword(); //< returns saveGroupPassword
		bool getUseSingleDes(); //< returns useSingleDes
		bool getUseIkeGroup(); //< returns useIkeGroup
		bool getUsePerfectForwardSecrety(); //< returns usePerfectSecrecy
		bool getUseNtDomainName(); //< returns useNtDomainName
		bool getUseApplicationVersion(); //< returns useApplicationVersion
		bool getUseLocalPort(); //< returns useLocalPort
		bool getUseAdvancedSettings(); //< returns useAdvancedSettings
		bool getUseGlobalIpsecSecret(); //< returns useGlobalIpsecSecret
		bool getDoPingIP(); //< returns doPingIP
		int getLocalPort(); //< returns LocalPort
		QString getPingHostIP(); //< returns last part of IP (host) for ping test
		QString getPrivateKeyPass(); //< returns PrivateKeyPass
		QString getPrivateKey(); //< returns PrivateKey
		bool getUseSpecialRemoteID(); //< returns useSpecialRemoteID

		/**
		 *   Sets the connection type
		 * @param type The type
		 */
		void setConnectionType( ConnectionType type );

		/**
		 *   Sets the name of the profile
		 * @param Name The new name
		 */
		void setName( QString Name );

		/**
		 *   Sets the IP/DNS name of the gateway
		 * @param Gateway the IP/DNS name
		 */
		void setGateway( QString Gateway );

		/**
		 *   Sets the remote ID (cisco)
		 * @param ID The ID
		 */
		void setID( QString ID );

		/**
		 *   Sets the group password (cisco)
		 * @param GroupPassword The password
		 */
		void setGroupPassword( QString GroupPassword );

		/**
		 *   Sets the username (cisco)
		 * @param Username The username
		 */
		void setUserName( QString Username );

		/**
		 *   Sets the user password
		 * @param UserPassword The password
		 */
		void setUserPassword( QString UserPassword );

		/**
		 *   Sets if user password should be stored or not
		 * @param saveUserPassword True if password should be stored
		 */
		void setSaveUserPassword( bool saveUserPassword );

		/**
		 *  Sets if group password should be stored or not
		 * @param saveGroupPassword True if password should be stored
		 */
		void setSaveGroupPassword( bool saveGroupPassword );

		/**
		 *   Sets the IKE group (cisco)
		 * @param IkeGroup The group
		 */
		void setIkeGroup( QString IkeGroup );

		void setPerfectForwardSecrety( QString PerfectForwardSecrecy );
		void setNtDomainName( QString Name );
		void setApplicationVersion( QString version );
		void setUseSingleDes( bool useSingleDes );
		void setLocalPort( int port );
		void setUseIkeGroup( bool useIkeGroup );
		void setUsePerfectForwardSecrety( bool usePerfectForwardSecrety );
		void setUseNtDomainName( bool useNtDomainName );
		void setUseApplicationVersion( bool useApplicationVersion );
		void setUseLocalPort( bool useLocalPort );
		void setUseAdvancedSettings( bool useAdvancedSettings );
		void setUseGlobalIpsecSecret( bool useGlobalIpsecSecret );
		void setRemoteNet( QString RemoteNet );
		void setPreSharedKey( QString PreSharedKey );
		void setX509Certificate( QString x509Certificate );
		void setNetworkDevice( QString NetworkDevice );
		void setCertPath( QString CertPath );
		void setAuthType( AuthenticationType authtype );
		void setDoPingIP( bool pingIP );
		void setPingHostIP( QString pingHostIP );
		void setPrivateKeyPass( QString PrivateKeyPass );
		void setPrivateKey( QString PrivateKey );
		void setUseSpecialRemoteID( bool useSpecialRemoteID );
		void setSpecialRemoteID( QString SpecialRemoteID );


	protected:
		QString Name; //< Name of the profile
		QString Gateway; //< IP/DNS name of the gateway
		QString ID; //< ID of remote gateway
		QString GroupPassword; //< Group password (cisco)
		QString Username; //< Username (cisco)
		QString UserPassword; //< User password (cisco)
		QString IkeGroup; //< DH group for PFS (cisco)
		QString PerfectForwardSecrecy; //< Type of PFS (cisco)
		QString NtDomainName; //< NT domain name for authentication (cisco)
		QString ApplicationVersion; //< Userdefinied application version (cisco)
		QString RemoteNet; //< Remote network of VPN
		QString PreSharedKey; //< Pre shared key (freeswan/racoon)
		QString x509Certificate; //< Path to X.509 certificate file
		QString NetworkDevice; //< Network device
		QString PrivateKeyPass; //< Password for private key
		QString PrivateKey; //< Path to private key file
		QString CertPath; //< Global certificate path
		QString AuthType; //< Authentication type (cert/psk)
		QString pingHostIP; //< IP of the host which should be pinged
		QString SpecialRemoteID; //< Userdefinied ID of remote gateway
		bool saveUserPassword; //< True if the user password should be stored (cisco)
		bool saveGroupPassword; //< True if the group password should be stored (cisco)
		bool useAdvancedSettings; //< True if advanced settings should be used (cisco)
		bool useIkeGroup; //< True if userdefinied IKE group should be used
		bool usePerfectForwardSecrety; //< True if userdefined PFS should be used
		bool useNtDomainName; //< True if NT domain name should eb used for authentication
		bool useApplicationVersion; //< True if userdefinied application version should be used
		bool useLocalPort; //< True if userdefinied local port should be used
		bool useSingleDes; //< True if DES instead of 3DES should be used
		bool useGlobalIpsecSecret; //< True if global group password from /etc/vpnc.conf should be used
		bool doPingIP; //< True if pingHostIP should be pinged after connection is established
		bool useSpecialRemoteID; //< True if userdefinied ID of remote gateway instead of certificate issuer should be used
		int LocalPort; //< Userdefinied local port
		ConnectionType connectiontype; //< Type of the connection (cisco, freeswan, racoon, etc.)
		AuthenticationType authtype; //< Type of authentication (cert, psk)

};

#endif
